/***************************************************************************
 *   Copyright (C) 2008 by S. MANKOWSKI / G. DE BURE skrooge@mankowski.fr  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>  *
 ***************************************************************************/
/** @file
* This file defines classes SKGImportXhb.
*
* @author Stephane MANKOWSKI / Guillaume DE BURE
 */
#include "skgimportxhb.h"
#include "skgtraces.h"
#include "skgservices.h"
#include "skgbankincludes.h"
#include "skgobjectbase.h"
#include "skgimportexportmanager.h"
#include "skgpayeeobject.h"

#include <klocale.h>
#include <kfilterdev.h>

#include <QDomDocument>

SKGError SKGImportXhb::importFile(SKGImportExportManager* iImporter, SKGDocumentBank* iDocument)
{
    if(!iImporter || !iDocument) return SKGError(ERR_ABORT, i18nc("Error message", "Invalid parameters"));
    SKGError err;
    SKGTRACEINRC(2, "SKGImportXhb::importFile", err);
    QString iFileName = iImporter->getFileName();
    SKGTRACEL(10) << "Input filename=" << iFileName << endl;

    //Initialisation
    //Open file
    QIODevice* file = KFilterDev::deviceForFile(iFileName, "application/x-gzip");
    if(!file->open(QIODevice::ReadOnly)) {
        err.setReturnCode(ERR_INVALIDARG);
        err.setMessage(i18nc("Error message",  "Open file '%1' failed", iFileName));
    } else {
        QDomDocument doc;

        //Set the file without uncompression
        QString errorMsg;
        int errorLine = 0;
        int errorCol = 0;
        bool contentOK = doc.setContent(file->readAll(), &errorMsg, &errorLine, &errorCol);
        file->close();

        //Get root
        QDomElement docElem = doc.documentElement();

        if(!contentOK) {
            err.setReturnCode(ERR_ABORT);
            err.setMessage(i18nc("Error message",  "%1-%2: '%3'", errorLine, errorCol, errorMsg));
        }


        if(!contentOK) {
            err.addError(ERR_INVALIDARG, i18nc("Error message",  "Invalid XML content in file '%1'", iFileName));
        } else {
            err = iDocument->beginTransaction("#INTERNAL#", 4);

            QMap<QString, SKGAccountObject> mapIdAccount;
            QMap<QString, SKGCategoryObject> mapIdCategory;
            QMap<QString, SKGPayeeObject> mapIdPayee;

            SKGUnitObject unit;
            if(err.isSucceeded()) err = iImporter->getDefaultUnit(unit);

            //Step 1-Create accounts
            if(err.isSucceeded()) {
                QDomNodeList accountList = docElem.elementsByTagName("account");
                int nb = accountList.count();
                err = iDocument->beginTransaction("#INTERNAL#", nb);
                for(int i = 0; err.isSucceeded() && i < nb; ++i) {
                    //Get account object
                    QDomElement account = accountList.at(i).toElement();

                    //Create the bank
                    SKGBankObject bank(iDocument);
                    QString bname = getAttribute(account,  "bankname");
                    if(bname.isEmpty()) bname = "HOMEBANK";
                    if(err.isSucceeded()) err = bank.setName(bname);
                    if(err.isSucceeded()) err = bank.save();

                    //Creation of the account
                    SKGAccountObject accountObj;
                    if(err.isSucceeded()) err = bank.addAccount(accountObj);
                    if(err.isSucceeded()) err = accountObj.setName(getAttribute(account,  "name"));
                    if(err.isSucceeded()) err = accountObj.setNumber(getAttribute(account,  "number"));
                    if(err.isSucceeded()) {
                        QString flags = getAttribute(account,  "flags");
                        err = accountObj.setClosed(flags == "2" ||  flags == "3");
                    }
                    if(err.isSucceeded()) err = accountObj.save();
                    if(err.isSucceeded()) {
                        accountObj.setClosed(false); //To be sure that the modification is possible. NOT SAVED
                        err = accountObj.setInitialBalance(SKGServices::stringToDouble(getAttribute(account,  "initial")), unit);
                    }

                    mapIdAccount[getAttribute(account,  "key")] = accountObj;

                    if(err.isSucceeded()) err = iDocument->stepForward(i + 1);
                }

                if(err.isSucceeded()) err = iDocument->endTransaction(true);
                else  iDocument->endTransaction(false);
            }
            if(err.isSucceeded()) err = iDocument->stepForward(1);

            //Step 2-Get payees
            if(err.isSucceeded()) {
                QDomNodeList partyList = docElem.elementsByTagName("pay");
                int nb = partyList.count();
                err = iDocument->beginTransaction("#INTERNAL#", nb);
                for(int i = 0; err.isSucceeded() && i < nb; ++i) {
                    //Get payee object
                    QDomElement party = partyList.at(i).toElement();
                    SKGPayeeObject payeeObject;
                    err = SKGPayeeObject::createPayee(iDocument, getAttribute(party,  "name"), payeeObject);
                    mapIdPayee[getAttribute(party,  "key")] = payeeObject;

                    if(err.isSucceeded()) err = iDocument->stepForward(i + 1);
                }

                if(err.isSucceeded()) err = iDocument->endTransaction(true);
                else  iDocument->endTransaction(false);
            }
            if(err.isSucceeded()) err = iDocument->stepForward(2);

            //Step 3-Create categories
            if(err.isSucceeded()) {
                QDomNodeList categoryList = docElem.elementsByTagName("cat");
                int nb = categoryList.count();
                err = iDocument->beginTransaction("#INTERNAL#", 2 * nb);
                for(int j = 0; err.isSucceeded() && j < 2; ++j) {
                    for(int i = 0; err.isSucceeded() && i < nb; ++i) {
                        //Get account object
                        QDomElement category = categoryList.at(i).toElement();

                        //Creation of the category
                        bool catCreated = false;
                        SKGCategoryObject catObj(iDocument);
                        QString parent = getAttribute(category,  "parent");
                        if(parent == "0" && j == 0) {
                            if(err.isSucceeded()) err = catObj.setName(getAttribute(category,  "name"));
                            if(err.isSucceeded()) err = catObj.save();

                            mapIdCategory[getAttribute(category,  "key")] = catObj;
                            catCreated = true;

                        } else if(parent != "0" && j == 1) {
                            SKGCategoryObject catParentObj = mapIdCategory[parent];
                            if(err.isSucceeded()) err = catObj.setName(getAttribute(category,  "name"));
                            if(err.isSucceeded()) err = catObj.setParentCategory(catParentObj);
                            if(err.isSucceeded()) err = catObj.save();

                            mapIdCategory[getAttribute(category,  "key")] = catObj;
                            catCreated = true;
                        }

                        //Creation of the attached budget
                        if(err.isSucceeded() && catCreated) {
                            QString b0 = getAttribute(category,  "b0");
                            QString b1 = getAttribute(category,  "b1");
                            if(!b0.isEmpty() || !b1.isEmpty()) {
                                for(int m = 1; err.isSucceeded() && m <= 12; ++m) {
                                    SKGBudgetObject budget(iDocument);

                                    int year = QDate::currentDate().year();
                                    if(err.isSucceeded()) err = budget.setCategory(catObj);
                                    if(err.isSucceeded()) err = budget.setYear(year);
                                    if(err.isSucceeded()) err = budget.setMonth(m);
                                    if(err.isSucceeded()) err = budget.setBudgetedAmount(SKGServices::stringToDouble(getAttribute(category,  'b' % SKGServices::intToString(b0.isEmpty() ? m : 0))));
                                    if(err.isSucceeded()) err = budget.save();
                                }
                            }
                        }

                        if(err.isSucceeded()) err = iDocument->stepForward(nb * j + i + 1);
                    }
                }

                if(err.isSucceeded()) err = iDocument->endTransaction(true);
                else  iDocument->endTransaction(false);
            }
            if(err.isSucceeded()) err = iDocument->stepForward(3);

            //Step 4-Create transaction
            if(err.isSucceeded()) {
                err = iDocument->beginTransaction("#INTERNAL#", 2);
                for(int j = 1; err.isSucceeded() && j <= 2; ++j) {
                    QDomNodeList transactionList = docElem.elementsByTagName(j == 1 ? "ope" : "fav");
                    int nb = transactionList.count();
                    err = iDocument->beginTransaction("#INTERNAL#", nb);
                    for(int i = 0; err.isSucceeded() && i < nb; ++i) {
                        //Get account object
                        QDomElement transaction = transactionList.at(i).toElement();

                        //Get attributes
                        QString idAccount = getAttribute(transaction,  "account");
                        QString dst_account = getAttribute(transaction,  "dst_account");
                        if(idAccount != "0") {
                            SKGAccountObject account = mapIdAccount[idAccount];
                            QDate date = QDate(1, 1, 1).addDays(1 + SKGServices::stringToInt(getAttribute(transaction,  j == 1 ? "date" : "nextdate")));
                            QString mode = getAttribute(transaction,  "paymode");
                            if(mode == "1") mode = i18nc("Noun: type of payement", "Credit card");
                            else if(mode == "2") mode = i18nc("Noun: type of payement", "Check");
                            else if(mode == "3") mode = i18nc("Noun: type of payement", "Cash");
                            else if(mode == "4" || mode == "5") mode = i18nc("Noun: type of payement", "Transfer");
                            else mode = i18nc("Noun: type of payement", "Other");
                            QString comment = QString(getAttribute(transaction,  "wording") % ' ' % getAttribute(transaction,  "info")).trimmed();
                            int flags = SKGServices::stringToInt(getAttribute(transaction,  "flags"));
                            double amount = SKGServices::stringToDouble(getAttribute(transaction,  "amount"));
                            SKGOperationObject::OperationStatus status = (flags == 1 ||  flags == 3 ? SKGOperationObject::CHECKED : SKGOperationObject::NONE);
                            SKGPayeeObject payee = mapIdPayee[ getAttribute(transaction,  "payee")];
                            SKGCategoryObject category = mapIdCategory[getAttribute(transaction,  "category")];

                            //Creation of the operation
                            SKGOperationObject opObj;
                            if(err.isSucceeded()) err = account.addOperation(opObj);
                            if(err.isSucceeded()) err = opObj.setDate(date);
                            if(err.isSucceeded()) err = opObj.setUnit(unit);
                            if(err.isSucceeded()) err = opObj.setPayee(payee);
                            if(err.isSucceeded()) err = opObj.setMode(mode);

                            if(err.isSucceeded()) err = opObj.setComment(comment);
                            if(err.isSucceeded()) err = opObj.setTemplate(j == 2);
                            if(err.isSucceeded()) err = opObj.setImported(true);
                            if(err.isSucceeded()) err = opObj.setImportID("HXB-" % SKGServices::intToString(i));
                            if(err.isSucceeded()) err = opObj.setStatus(status);
                            if(err.isSucceeded()) err = opObj.save();

                            SKGSubOperationObject subObj;
                            if(err.isSucceeded()) err = opObj.addSubOperation(subObj);
                            if(err.isSucceeded()) err = subObj.setCategory(category);
                            if(err.isSucceeded()) err = subObj.setComment(comment);
                            if(err.isSucceeded()) err = subObj.setQuantity(amount);
                            if(err.isSucceeded()) err = subObj.save();

                            //Transfert
                            /*if(dst_account != idAccount && dst_account != "0") {
                                SKGAccountObject account = mapIdAccount[dst_account];
                                SKGOperationObject op2Obj;
                                if(err.isSucceeded()) err = account.addOperation(op2Obj);
                                if(err.isSucceeded()) err = op2Obj.setDate(date);
                                if(err.isSucceeded()) err = op2Obj.setUnit(unit);
                                if(err.isSucceeded()) err = op2Obj.setPayee(payee);
                                if(err.isSucceeded()) err = op2Obj.setMode(mode);

                                if(err.isSucceeded()) err = op2Obj.setComment(comment);
                                if(err.isSucceeded()) err = op2Obj.setTemplate(j == 2);
                                if(err.isSucceeded()) err = op2Obj.setImported(true);
                                if(err.isSucceeded()) err = op2Obj.setImportID("HXB-" % SKGServices::intToString(i) % "_TR");
                                if(err.isSucceeded()) err = op2Obj.setStatus(status);
                                if(err.isSucceeded()) err = op2Obj.save();

                                SKGSubOperationObject subObj;
                                if(err.isSucceeded()) err = op2Obj.addSubOperation(subObj);
                                if(err.isSucceeded()) err = subObj.setCategory(category);
                                if(err.isSucceeded()) err = subObj.setComment(comment);
                                if(err.isSucceeded()) err = subObj.setQuantity(-amount);
                                if(err.isSucceeded()) err = subObj.save();

                                if(err.isSucceeded()) err = op2Obj.setGroupOperation(opObj);
                                if(err.isSucceeded()) err = op2Obj.save();
                            }*/

                            //Scheduled operations
                            if(j == 2) {
                                //Creation of scheduled operation
                                SKGRecurrentOperationObject recu;
                                if(err.isSucceeded()) err = opObj.addRecurrentOperation(recu);
                                int limit = SKGServices::stringToInt(getAttribute(transaction,  "limit"));
                                if(err.isSucceeded()) err = recu.timeLimit(flags & (1 << 7));
                                if(err.isSucceeded()) err = recu.setTimeLimit(limit);
                                int punit = SKGServices::stringToInt(getAttribute(transaction,  "unit"));
                                int pstep = SKGServices::stringToInt(getAttribute(transaction,  "every"));
                                SKGRecurrentOperationObject::PeriodUnit p = SKGRecurrentOperationObject::MONTH;
                                if(punit == 0) p = SKGRecurrentOperationObject::DAY;
                                else if(punit == 1) {
                                    p = SKGRecurrentOperationObject::DAY;
                                    pstep *= 7;
                                } else if(punit == 3) p = SKGRecurrentOperationObject::YEAR;
                                if(err.isSucceeded()) err = recu.setPeriodIncrement(pstep);
                                if(err.isSucceeded()) err = recu.setPeriodUnit(p);
                                if(err.isSucceeded()) err = recu.warnEnabled(flags & (1 << 5));
                                if(err.isSucceeded()) err = recu.autoWriteEnabled(flags & (1 << 2));
                                if(err.isSucceeded()) err = recu.save();
                                //#define OF_VALID        (1<<0)
                                //#define OF_INCOME       (1<<1)
                                //#define OF_AUTO         (1<<2)
                                //#define OF_ADDED        (1<<3)
                                //#define OF_CHANGED      (1<<4)
                                //#define OF_REMIND       (1<<5)
                                //#define OF_CHEQ2        (1<<6)
                                //#define OF_LIMIT        (1<<7)
                            }
                        }

                        if(err.isSucceeded() && i % 100 == 0) err = iDocument->executeSqliteOrder("ANALYZE");
                        if(err.isSucceeded()) err = iDocument->stepForward(i + 1);
                    }

                    if(err.isSucceeded()) err = iDocument->endTransaction(true);
                    else  iDocument->endTransaction(false);

                    if(err.isSucceeded()) err = iDocument->stepForward(j);
                }

                if(err.isSucceeded()) err = iDocument->endTransaction(true);
                else  iDocument->endTransaction(false);
            }
            if(err.isSucceeded()) err = iDocument->stepForward(4);

            if(err.isSucceeded()) err = iDocument->endTransaction(true);
            else  iDocument->endTransaction(false);

            if(err.isSucceeded()) err = iDocument->executeSqliteOrder("ANALYZE");
        }
    }

    delete file;
    return err;
}


QString SKGImportXhb::getAttribute(const QDomElement& iElement, const QString& iAttribute)
{
    QString val = iElement.attribute(iAttribute);
    if(val == "(null)") val = "";
    return val;
}

#include "skgimportxhb.moc"
